unit fQuoteDef;

interface

uses
  Windows,
  Classes,
  SysUtils,
  ExtCtrls,
  Forms,
  Math,
  NMHttp;

const

  sfName  =0; //0:W
  sfTime  =1; //1:ɶ
  sfDeal  =2; //2:
  sfBid   =3; //3:Ri
  sfAsk   =4; //4:X
  sfDiff  =5; //5:^
  sfVol   =6; //6:i
  sfPreC  =7; //7:Q
  sfOpen  =8; //8:}L
  sfHigh  =9; //9:̰
  sfLow   =10;//10:̧C

type
{ TStockFieldSet }
  TStockFieldSet = set of sfName..sfLow;

{ TArrayOfString }
  TArrayOfString = array of string;

{ TStockRec }
  PStockRec = ^TStockRec;
  TStockRec = packed record
    LastUpdate: TDateTime;
    LastInterval: Extended;
    UpdatedSet: TStockFieldSet;
    InitUpdated: Boolean;
    ID: string;
    Ref: Single;
    Data: TArrayOfString;
  end;

{ TStockObj }
  TStockObj = class(TPersistent)
  private
    F_Rec: TStockRec;
    function Get_Rec: PStockRec;
    function GetStockData(StkID: string; var Ref: Single; var A: TArrayOfString): Boolean;
    function UpdateStockData: Boolean;
  public
    constructor Create(ID: string);
    procedure ResetTo(ID: string);
    property Rec: PStockRec read Get_Rec;
  end;

{ TStockObjList }

  TObjUpdateEvent = procedure(Sender: TObject; Index: Integer; Obj: TStockObj) of object;

  TStockObjList = class(TPersistent)
  private
    F_List: TStringList;
    F_OnUpdate: TObjUpdateEvent;
    F_Timer: TTimer;
    function GetCount: Integer;
    function GetObj(Index: Integer): TStockObj;
    procedure DO_OnTimer(Sender: TObject);
  protected
    procedure Clear;
    procedure Load;
    procedure Save;
  public
    constructor Create;
    destructor Destroy; override;
    procedure UpdateStockData(JustDoIt: Boolean);
    procedure UpdateStockData__(Index: Integer; O: TStockObj);
    property Count: Integer read GetCount;
    property Obj[Index: Integer]: TStockObj read GetObj; default;
    property OnUpdate: TObjUpdateEvent read F_OnUpdate write F_OnUpdate;
    property List: TStringList read F_List;
  end;

  function MyStrToFloat(S: string): Single;
  function InTradeTimeInterval: Boolean;

const
  FN_AVAILABLE = 'AVAILABLE.TXT';
var
  MAX_STOCK_COUNT: Integer = 10;
  DATA_UPDATE_CANCEL: Boolean = False;
  IS_DATA_UPDATING: Boolean = False;
  IS_DATA_GETTING: Boolean = False;
  IS_HOLIDAY: Boolean = False;
  
implementation

const
  FN_STOCKLIST = 'STOCKLIST.TXT';
  INTERVAL_DIFF = 1/86400;
  MIN_INTERVAL = 8/86400;
  MAX_INTERVAL = 25/86400;
  MARKET_OPEN = '08:55:00';
  MARKET_CLOSE = '13:35:00';

function MyStrToFloat(S: string): Single;
var
  I,Sign: Integer;
  mS: string;
begin
  Result := 0;

  Sign := 1;
  if Pos('',S)>0 then Sign := -1
  else if Pos('',S)>0 then Sign := -1;

  mS := '';
  for I := 1 to Length(S) do
  begin
    if S[I] in ['0'..'9','.'] then
      mS := mS + S[I];
  end;

  if mS<>'' then Result := Sign*StrToFloatDef(mS,Result);

end;

function InTradeTimeInterval: Boolean;
var
  dt: string;
begin
  dt := FormatDateTime('hh:nn',Now);
  Result :=
    not IS_HOLIDAY and
    (dt>=MARKET_OPEN) and
    (dt<MARKET_CLOSE);
end;

{ TStockObj }

constructor TStockObj.Create(ID: string);
begin
  inherited Create;
  ResetTo(ID);
end;

function TStockObj.GetStockData(
    StkID: string;
    var Ref: Single;
    var A: TArrayOfString): Boolean;
var
  I,J,K: Integer;
  P1,P2: Integer;
  AID,URL,Key,S,SSS: string;
  L: TStringList;
  HHH: TNMHTTP;
  X: Single;
begin
  A := nil;
  Ref := 0;
  Result := False;
  AID := Trim(StkID);
  try
    IS_DATA_GETTING := True;
    try
//
      HHH := TNMHTTP.Create(nil);
      try
        URL := Format('http://tw.stock.yahoo.com/q/q?s=%s',[AID]);
        try
          HHH.Get(URL);
          S := HHH.Body;
        finally
          HHH.Disconnect;
        end;
      finally
        FreeAndNil(HHH);
      end;
//*)      
    finally
      IS_DATA_GETTING := False;
    end;
  except
    EXIT;
  end;

  if S='' then EXIT;

  L := TStringList.Create;
  try
    L.Text := S;
    J := -1;
    SSS := Format('href="/q/bc?s=%s">%s',[AID,AID]);
    for I := 0 to L.Count-1 do
    begin
      if Pos(SSS,L[I])>0 then
      begin
        J := I;
        Break;
      end;
    end;

    if J<>-1 then
    begin
      SetLength(A,11);
      for I := 0 to Length(A)-1 do
      begin
        K := I+J;
        if K>L.Count-1 then Break;
        S := Trim(L[K]);
        case I of
          0: Key  := SSS;
          2: Key  := '<td align="center" bgcolor="#FFFfff" nowrap><b>';
          5:
            begin
              P1 := Pos('',S);
              if P1<1 then P1 := Pos('',S);
              if P1<1 then P1 := Pos('',S);
              if P1<1 then P1 := Pos('',S);
              if P1>1 then A[I] := Trim(Copy(S,P1,Length(S)-P1+1));
              Continue;
            end;
          else Key:= '<td align="center" bgcolor="#FFFfff" nowrap>';
        end;
        P1 := Pos(Key,S);
        P2 := Pos('</',S);
        P1 := P1+Length(Key);
        A[I] := Trim(Copy(S,P1,P2-P1));
      end;

      if Length(A[0])=4 then
      begin
        A[0] := Copy(A[0],1,2)+'@'+Copy(A[0],3,2);
      end;

      if Ref=0 then
      begin
        X := MyStrToFloat(A[2]);
        if X>0 then
        begin
          Ref := X-MyStrToFloat(A[5]);
        end;
      end;

    end;
  finally
    FreeAndNil(L);
  end;
  Result := A<>nil;
end;

function TStockObj.Get_Rec: PStockRec;
begin
  Result := @F_Rec;
end;

procedure TStockObj.ResetTo(ID: string);
var
  I: Integer;
begin
  F_Rec.ID := ID;
  F_Rec.LastUpdate := 0;
  F_Rec.LastInterval := 0;
  F_Rec.UpdatedSet := [];
  F_Rec.InitUpdated := False;
  F_Rec.Ref := 0;
  if Length(F_Rec.Data)<>11 then SetLength(F_Rec.Data,11);
  for I := 0 to Length(F_Rec.Data)-1 do F_Rec.Data[I] := '';
end;

function TStockObj.UpdateStockData: Boolean;
var
  I: Integer;
  A: TArrayOfString;
  mRef: Single;
  E: Extended;
begin
  Result := False;
  with F_Rec do
  begin
    if not InTradeTimeInterval and InitUpdated then EXIT;

    E := Now-LastUpdate;

    if E>LastInterval then
    begin
      LastUpdate := Now;
      if GetStockData(ID,mRef,A) then
      begin
        if mRef>0 then Ref := mRef;
        for I := 0 to Length(A)-1 do
        begin
          if IS_DATA_GETTING then
          begin
            EXIT;
          end;
          if Data[I]=A[I] then Continue;
          UpdatedSet := UpdatedSet + [I];
          Data[I] := A[I];
        end;

        if UpdatedSet=[] then
        begin
          LastInterval := Min(MAX_INTERVAL,LastInterval+INTERVAL_DIFF);
        end
        else begin
          InitUpdated := True;
          LastInterval := Max(MIN_INTERVAL,LastInterval-INTERVAL_DIFF);
          Result := True;
          
        end;

      end;
    end;
  end;
end;

{ TStockObjList }

procedure TStockObjList.Clear;
var
  I: Integer;
begin
  for I := 0 to F_List.Count-1 do
    F_List.Objects[I].Free;
  F_List.Clear;
end;

constructor TStockObjList.Create;
begin
  inherited Create;
  F_List := TStringList.Create;
  Load();

  F_Timer := TTimer.Create(nil);
  F_Timer.Interval := 5000;
  F_Timer.OnTimer := DO_OnTimer;
  F_Timer.Enabled := True;
end;

destructor TStockObjList.Destroy;
begin
  F_Timer.Enabled := False;
  Save();
  Clear();
  FreeAndNil(F_List);
  FreeAndNil(F_Timer);
  inherited;
end;

procedure TStockObjList.DO_OnTimer(Sender: TObject);
begin
  F_Timer.Enabled := False;
  try
    UpdateStockData(False);
  finally
    F_Timer.Enabled := true;
  end;
end;

function TStockObjList.GetCount: Integer;
begin
  if F_List<>nil
  then Result := F_List.Count
  else Result := 0;
end;

function TStockObjList.GetObj(Index: Integer): TStockObj;
begin
  if (Index>-1) and (Index<F_List.Count)
  then Result := TStockObj(F_List.Objects[Index])
  else Result := nil;
end;

procedure TStockObjList.Load;
var
  I,CCC: Integer;
  L: TStringList;
  O: TStockObj;
  AID: string;
begin
  Clear;
  if FileExists(FN_STOCKLIST) then
  begin
    L := TStringList.Create;
    try
      L.LoadFromFile(FN_STOCKLIST);
      CCC := Min(L.Count,MAX_STOCK_COUNT);
      for I := 0 to CCC-1 do
      begin
        AID := Trim(L[I]);
        if AID = '' then Continue;
        O := TStockObj.Create(AID);
        F_List.AddObject(O.F_Rec.ID,O);
      end;
    finally
      FreeAndNil(L);
    end;
  end;

end;

procedure TStockObjList.Save;
var
  I: Integer;
  L: TStringList;
  O: TStockObj;
begin
  if F_List.Count = 0
  then DeleteFile(FN_STOCKLIST)
  else begin
    L := TStringList.Create;
    try
      for I := 0 to F_List.Count-1 do
      begin
        O := Obj[I];
        if O=nil then Continue;
        L.Add(Trim(O.Rec.ID));
      end;
      L.SaveToFile(FN_STOCKLIST);
    finally
      FreeAndNil(L);
    end;
  end;
end;


procedure TStockObjList.UpdateStockData(JustDoIt: Boolean);
var
  I: Integer;
  O: TStockObj;
begin
  if DATA_UPDATE_CANCEL then EXIT;
  if IS_DATA_UPDATING then EXIT;

  IS_DATA_UPDATING := True;
  TRY
    for I := 0 to F_List.Count-1 do
    begin

      if DATA_UPDATE_CANCEL then
      begin
        IS_DATA_UPDATING := False;
        EXIT;
      end;

      O := TStockObj(F_List.Objects[I]);
      if O=nil then Continue;
      if JustDoIt then O.Rec.LastUpdate := 0;
      if O.UpdateStockData and (O.Rec.UpdatedSet<>[]) then
      begin
        if Assigned(F_OnUpdate) then F_OnUpdate(Self,I,O);
        O.F_Rec.UpdatedSet := [];
      end;
      Application.ProcessMessages;
      Sleep(50);
    end;
  FINALLY
    IS_DATA_UPDATING := False;
  END;

end;

procedure TStockObjList.UpdateStockData__(Index: Integer; O: TStockObj);
begin
  O.Rec.LastUpdate := 0;
  if O.UpdateStockData and (O.Rec.UpdatedSet<>[]) then
  begin
    if Assigned(F_OnUpdate) then F_OnUpdate(Self,Index,O);
    O.F_Rec.UpdatedSet := [];
  end;
end;

end.


