unit fQuoteMain;

interface

uses
  Windows,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Math,
  ExtCtrls,
  Grids,
  BaseGrid,
  AdvGrid,
  fQuoteDef,
  fDlgYesNo,
  fDlgMenu,
  fDlgStockSel;

type
{ TfrmWatchMain }
  TfrmWatchMain = class(TForm)
    Panel1: TPanel;
    G_: TAdvStringGrid;
    tmrMain: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure G_Resize(Sender: TObject);
    procedure tmrMainTimer(Sender: TObject);

    procedure G_GetAlignment(
        Sender: TObject;
        ARow, ACol: Integer;
        var HAlign: TAlignment;
        var VAlign: TVAlignment);

    procedure G_GetCellColor(
        Sender: TObject;
        ARow, ACol: Integer;
        AState: TGridDrawState;
        ABrush: TBrush;
        AFont: TFont);
    procedure Panel1DblClick(Sender: TObject);
    procedure G_RowChanging(Sender: TObject; OldRow, NewRow: Integer; var Allow: Boolean);
    procedure DO_OnUpdate(Sender: TObject; Index: Integer; Obj: TStockObj);
    procedure InitialGrid;
    procedure G_MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  protected
    DISPLAY_MODE: Integer;//Display Mode
    DISPLAY_SET: TStockFieldSet;
    F_StockList: TStockObjList;
    F_Available: TStringList;
    procedure DO_MAIN_MENU(ARow: Integer);
    procedure DO_STOCK_SEL;
    procedure DO_UpdateALL;
    procedure DO_SWITCH_FIELDSET;
  end;

var
  frmWatchMain: TfrmWatchMain;

implementation

{$R *.dfm}

const
  MAP_T: array[0..2,0..5] of Integer = (
    (sfName,sfBid,sfAsk,sfDeal,sfDiff,sfVol),
    (sfName,sfDeal,sfDiff,sfOpen,sfHigh,sfLow),
    (sfName,sfDeal,sfDiff,sfVol,sfHigh,sfLow));

  MAP_S: array[0..2] of string = (
    ',Ri,X,,"^",q',
    ',,"^",}L,̰,̧C',
    ',,"^",q,̰,̧C');

procedure TfrmWatchMain.FormCreate(Sender: TObject);
begin
  IS_HOLIDAY := (Trunc(Now) mod 7) in [0,1];
  DISPLAY_MODE := 0;
  G_.Font.Name := 'ARIAL';
  G_.ColCount := 6;
  G_.RowCount := MAX_STOCK_COUNT+1;
  G_.Row := 1;

  F_Available := TStringList.Create;
  F_Available.Sorted := True;
  if FileExists(FN_AVAILABLE) then
  begin
    F_Available.LoadFromFile(FN_AVAILABLE);
  end;

  F_StockList := TStockObjList.Create;
  F_StockList.OnUpdate := DO_OnUpdate;
  Show;
  Update;
  F_StockList.UpdateStockData(True);

end;

procedure TfrmWatchMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  f: TfrmDlgYesNo;
begin
  CanClose := False;
  if IS_DATA_UPDATING then EXIT;
  
  DATA_UPDATE_CANCEL := True;
  f := TfrmDlgYesNo.Create(Self);
  try
    CanClose := f.ShowModal=mrOK;
  finally
    FreeAndNil(f);
  end;

  if not CanClose then
  begin
    DATA_UPDATE_CANCEL := False;
  end;

end;

procedure TfrmWatchMain.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_ESCAPE: Close;
    VK_SPACE: DO_SWITCH_FIELDSET;
    VK_F5:
      if F_StockList<>nil then
      begin
        F_StockList.UpdateStockData(True);
      end;
  end;
end;

procedure TfrmWatchMain.G_Resize(Sender: TObject);
begin
  InitialGrid;
end;

procedure TfrmWatchMain.G_GetAlignment(Sender: TObject; ARow,
  ACol: Integer; var HAlign: TAlignment; var VAlign: TVAlignment);
var
  T: Integer;
begin
  VAlign := vtaCenter;
  if (ARow=0) or (ACol=0) then HAlign := taCenter
  else begin
    T := MAP_T[DISPLAY_MODE,ACol];
    case T of
      sfName,sfTime:  HAlign := taCenter;
      sfDiff:         HAlign := taLeftJustify;
      else            HAlign := taRightJustify;
    end;
  end;
end;

procedure TfrmWatchMain.DO_OnUpdate(Sender: TObject; Index: Integer; Obj: TStockObj);
var
  S: string;
  Col,Row: Integer;
  T: Integer;
begin
  with Obj.Rec^ do
  begin
    if UpdatedSet=[] then EXIT;
    if Index>=G_.RowCount-1 then EXIT;
    Row := Index+1;
    for Col := 0 to G_.ColCount-1 do
    begin
      if UpdatedSet=[] then Break;
      T := MAP_T[DISPLAY_MODE,Col];
      if not (T in UpdatedSet) then Continue;
      UpdatedSet := UpdatedSet-[];

      S := Trim(Data[T]);
      case T of
        sfName:;//PASS(ҨW)
        sfDiff: S := ' '+S;
        else    S := S+' ';
      end;
      G_.Cells[Col,Row] := S;
    end;
  end;

end;

procedure TfrmWatchMain.FormDestroy(Sender: TObject);
begin
  FreeAndNil(F_StockList);
  FreeAndNil(F_Available);
end;

procedure TfrmWatchMain.tmrMainTimer(Sender: TObject);
begin
  G_.Cells[0,0] := FormatDateTime('hh:nn',Now);
end;

procedure TfrmWatchMain.G_GetCellColor(Sender: TObject; ARow,
  ACol: Integer; AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
var
  S: string;
  V,mRef: Single;
  O: TStockObj;
  T: Integer;
begin
       if (ACol=0) and (ARow=0) then AFont.Name := 'ARIAL'
  else if (ACol=0) or  (ARow=0) then AFont.Name := 'з'
  else                               AFont.Name := 'ARIAL';

  T := MAP_T[DISPLAY_MODE,ACol];
  case ARow of
    0:
      case ACol of
        0:
        begin
          if InTradeTimeInterval
          then AFont.Color := clSkyBlue
          else AFont.Color := clGray;
          AFont.Height := Trunc(AFont.Height*Sqrt(4/2.9));
          AFont.Style := AFont.Style+[fsBold];
          if not IS_DATA_UPDATING
          then ABrush.Color := clBlack
          else begin
            if IS_DATA_GETTING
            then ABrush.Color := clMaroon
            else ABrush.Color := clNavy;
          end;
        end;
        else begin
          case T of
            sfDeal,sfDiff,sfVol: AFont.Color := clWhite;
            sfHigh: AFont.Color := clRed;
            sfLow:AFont.Color := clAqua;
            else AFont.Color := clGray;
          end;
        end;
      end;
    else begin
      case T of

        sfName:
          begin
            if ARow mod 2 = 0
            then AFont.Color := clLime
            else AFont.Color := clYellow;
          end;

        sfTime: AFont.Color := clSilver;

        sfDeal,sfBid,sfAsk,sfPreC,
        sfOpen,sfHigh,sfLow:
          begin
            O := F_StockList[ARow-1];
            if (O<>nil) and (O.Rec.Ref>0) then
            begin
              mRef := O.Rec.Ref;
              V := G_.Floats[ACol,ARow];
                    if V>mRef then AFont.Color := clRed
              else  if V<mRef then AFont.Color := clAqua
              else                 AFont.Color := clLime;
            end;
          end;
        sfDiff:
          begin
            S := G_.Cells[ACol,ARow];
                  if Pos('',S)>0 then AFont.Color := clRed
            else  if Pos('',S)>0 then AFont.Color := clAqua
            else  if Pos('',S)>0 then AFont.Color := clRed
            else  if Pos('',S)>0 then AFont.Color := clAqua
            else                        AFont.Color := clLime;
          end;
        sfVol: AFont.Color := clYellow;
      end;
    end;
  end;
end;

procedure TfrmWatchMain.InitialGrid;
var
  I: Integer;
begin
  with G_ do
  begin
    DefaultRowHeight := ClientHeight div RowCount - GridLineWidth;
    DefaultColWidth := ClientWidth div ColCount - GridLineWidth;
    Font.Height := Min(Trunc(DefaultRowHeight),Trunc(DefaultColWidth/3.75));
    ClearNormalCells;
    Rows[0].CommaText := MAP_S[DISPLAY_MODE];
    Cells[0,0] := FormatDateTime('hh:nn',Now);
  end;

  DISPLAY_SET := [];
  for I := 0 to Length(MAP_T[DISPLAY_MODE])-1 do
  begin
    DISPLAY_SET := DISPLAY_SET+[MAP_T[DISPLAY_MODE][I]];
  end;

end;

procedure TfrmWatchMain.DO_UpdateALL;
var
  I,Count: Integer;
  O: TStockObj;
begin
  InitialGrid;
  Count := F_StockList.Count;
  for I := 0 to Count-1 do
  begin
    O := F_StockList[I];
    if O<>nil then
    begin
      O.Rec.UpdatedSet := DISPLAY_SET;
      DO_OnUpdate(nil,I,O);
    end;
  end;
end;

procedure TfrmWatchMain.Panel1DblClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmWatchMain.DO_SWITCH_FIELDSET;
begin
  DISPLAY_MODE := (DISPLAY_MODE+1) mod 3;
  DO_UpdateALL;
end;

procedure TfrmWatchMain.G_RowChanging(Sender: TObject; OldRow,
  NewRow: Integer; var Allow: Boolean);
begin
  Allow := (NewRow>0) and (NewRow<G_.RowCount);
end;

procedure TfrmWatchMain.DO_MAIN_MENU(ARow: Integer);
var
  f: TfrmDlgMenu;
  rt: TRect;
  pt: TPoint;
begin
  if IS_DATA_UPDATING then EXIT;
  DATA_UPDATE_CANCEL := True;
  try
    f := TfrmDlgMenu.Create(Self);
    try
      if ARow=0 then
      begin
        rt := G_.CellRect(0,ARow);
        pt := G_.ClientToScreen(rt.TopLeft);
        f.Left := pt.X;
        f.Top := pt.Y;
        f.ListBox1.Items.CommaText := '{';
      end
      else begin
        GetCursorPos(pt);
(*
        rt := G_.CellRect(0,ARow);
        pt := G_.ClientToScreen(rt.TopLeft);
        f.Left := pt.X+G_.DefaultColWidth div 5;
        f.Top := pt.Y+G_.DefaultRowHeight;
*)
        f.Left := pt.X;
        f.Top := pt.Y;
        f.ListBox1.Items.CommaText := '󴫪Ѳ';
      end;
      f.ListBox1.ItemIndex := 0;
      f.Height := f.ListBox1.ItemHeight;
      if f.ShowModal=mrOK then
      begin
        case ARow of
          0: Close
          else DO_STOCK_SEL;
        end;
      end;
    finally
      FreeAndNil(f);
    end;
  finally
    DATA_UPDATE_CANCEL := False;
  end;

end;

procedure TfrmWatchMain.DO_STOCK_SEL;
var
  I: Integer;
  f: TfrmStockSel;
  rt: TRect;
  pt: TPoint;
  O: TStockObj;
begin
  if IS_DATA_UPDATING then EXIT;
  DATA_UPDATE_CANCEL := True;
  try
    f := TfrmStockSel.Create(Self,F_Available);
    try
      rt := G_.CellRect(0,0);
      pt := G_.ClientToScreen(rt.BottomRight);
      f.Left := pt.X+1;
      f.Top := pt.Y;
      if f.ShowModal=mrOK then
      begin
        if Length(f.SelectedID)=4 then
        begin
          I := G_.Row-1;
          O := F_StockList[G_.Row-1];
          if O<>nil then
          begin
            O.ResetTo(f.SelectedID);
          end
          else begin
            O := TStockObj.Create(f.SelectedID);
            I := F_StockList.List.AddObject(f.SelectedID,O);
          end;
          F_StockList.UpdateStockData__(I,O);
        end;
      end;
    finally
      FreeAndNil(f);
    end;
  finally
    DATA_UPDATE_CANCEL := False;
  end;

end;

procedure TfrmWatchMain.G_MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  ACol,ARow: Integer;
begin
  G_.MouseToCell(X,Y,ACol,ARow);
  if not (ssRight in Shift) then
  begin
    if (ARow=0) and (ACol>0) then DO_SWITCH_FIELDSET;
  end
  else begin
    if (ARow=0) and (ACol=0) then DO_MAIN_MENU(ARow)
    else if (ARow>0) then
    begin
      G_.Row := ARow;
      DO_MAIN_MENU(ARow);
    end;
  end;
end;

end.
